/* ========================================
 *
 * 
 *
 * ========================================
*/

#include <myEEPROM_1.h>

//#include <stdio.h> //debugging only


// EEPROM_SIZE - The size of the EEPROM memory space in bytes
// CYDEV_EE_SIZE – The size of the EEPROM memory space in bytes
// EEPROM_NUMBER_ROWS - The number of rows of EEPROM memory
// CYDEV_EEPROM_ROW_SIZE – The size of a row of the EEPROM in bytes
// CYDEV_EE_BASE – The base pointer of the EEPROM memory 

// Using the base pointer, individual bytes of the EEPROM memory can be read.
// To navigate to a specific row the base pointer must be incremented by the number of times
// of the size of the EEPROM row. 

// It can take as many as 20 ms to write to EEPROM. 

// defines ==================================================================

//'static' is required for multiple instances
static uint8 *PConfig;             //pointer to the config structure
static uint16 SIZEOF_CONFIG_BUFFER;//size of the config structure 
static uint8 config_init_flag = 0; //flag to indicate that config was initialized   

//char strMsg1[128];//output UART buffer - debug only

//===========================================================================
// For a byte array whose accumulated crc value is stored in *crc, computes
// resultant crc obtained by appending m to the byte array

static void crc8(uint8 *crc, uint8 m)
{
    // static table for CRC8-ATM (x^8 + x^2 + x + 1)
    static uint8 crc8_table[256] = {
       0,   7,  14,   9,  28,  27,  18,  21,  56,  63,                              
      54,  49,  36,  35,  42,  45, 112, 119, 126, 121,                              
     108, 107,  98, 101,  72,  79,  70,  65,  84,  83,                              
      90,  93, 224, 231, 238, 233, 252, 251, 242, 245,                              
     216, 223, 214, 209, 196, 195, 202, 205, 144, 151,                              
     158, 153, 140, 139, 130, 133, 168, 175, 166, 161,                              
     180, 179, 186, 189, 199, 192, 201, 206, 219, 220,                              
     213, 210, 255, 248, 241, 246, 227, 228, 237, 234,                              
     183, 176, 185, 190, 171, 172, 165, 162, 143, 136,                              
     129, 134, 147, 148, 157, 154,  39,  32,  41,  46,                              
      59,  60,  53,  50,  31,  24,  17,  22,   3,   4,                              
      13,  10,  87,  80,  89,  94,  75,  76,  69,  66,                              
     111, 104,  97, 102, 115, 116, 125, 122, 137, 142,                              
     135, 128, 149, 146, 155, 156, 177, 182, 191, 184,                              
     173, 170, 163, 164, 249, 254, 247, 240, 229, 226,                              
     235, 236, 193, 198, 207, 200, 221, 218, 211, 212,                              
     105, 110, 103,  96, 117, 114, 123, 124,  81,  86,                              
      95,  88,  77,  74,  67,  68,  25,  30,  23,  16,                              
       5,   2,  11,  12,  33,  38,  47,  40,  61,  58,                              
      51,  52,  78,  73,  64,  71,  82,  85,  92,  91,                              
     118, 113, 120, 127, 106, 109, 100,  99,  62,  57,                              
      48,  55,  34,  37,  44,  43,   6,   1,   8,  15,                              
      26,  29,  20,  19, 174, 169, 160, 167, 178, 181,                              
     188, 187, 150, 145, 152, 159, 138, 141, 132, 131,                              
     222, 217, 208, 215, 194, 197, 204, 203, 230, 225,                              
     232, 239, 250, 253, 244, 243
    }; 
    
    
    *crc = crc8_table[(*crc) ^ m];
    *crc &= 0xFF;
}

//===========================================================================

static uint8 GetCRC8( reg8 * PBuffer ) //gets CRC8 for byte buffer
{
    uint8  crc;
    uint16 index;//buffer length may be upto 16*64=1kb
 
    crc = myEEPROM_1_CRC_SEED_VAL; //initialize CRC
    for (index=0; index < SIZEOF_CONFIG_BUFFER; index++) //if CRC is last byte at (SIZEOF_CONFIG_BUFFER+1) 
    {
        crc8(&crc, PBuffer[index]);  
    }
    return crc;
}

//===========================================================================

uint8 myEEPROM_1_recallConfig(void) //recall buffer from EEPROM
{
    uint8 result = 0;
    
    reg8 * EEPROMRegPointer = (reg8 *) CYDEV_EE_BASE; //starting addr of the "0"-row
    uint8 tmpBuf[SIZEOF_CONFIG_BUFFER+1];   //temporary copy of Config +CRC byte
    uint8 * PtmpBuf = &(tmpBuf[0]);         //pointer to tmpBuf

    if (config_init_flag != 0) //if initialized correctly
    {
        myEEPROM_1_EEPROM_Start();
         
        uint16 index;   //buffer length may be upto 2048 bytes
        for(index = 0; index < SIZEOF_CONFIG_BUFFER+1; index++)
        {
            //PtmpBuf[index] = EEPROMRegPointer[index]; //copy EEPROM to temp buffer
            //tmpBuf[index] = EEPROMRegPointer[index]; // copy EEPROM to temp buffer
            tmpBuf[index] = EEPROMRegPointer[myEEPROM_1_EEPROM_CONFIG_LOCATION * SIZEOF_EEPROM_ROW + index]; // copy EEPROM to temp buffer
        }
        uint8 CRC = tmpBuf[SIZEOF_CONFIG_BUFFER];//if CRC is last byte
        
        //sprintf(strMsg1,"recallConfig: tmpBuf.CRC=%d, GetCRC8=%d\r\n", CRC, GetCRC8(PtmpBuf) );//debugging..
        //SW_Tx_UART_PutString(strMsg1);//debugging only
        
        // if CRC check requested -> copy data only if calculated CRC matches stored value
        // if CRC  not requested  -> copy data as-is
        if ( (myEEPROM_1_check_crc_flag & (CRC==GetCRC8(PtmpBuf)) )
          || !myEEPROM_1_check_crc_flag ) 
        {
            memcpy( PConfig, &tmpBuf, SIZEOF_CONFIG_BUFFER );//copy temp buffer to Config structure excluding last CRC byte
            result = 1;
        }
    
        myEEPROM_1_EEPROM_Stop(); //optional 
    }
    
    return result;
}

//===========================================================================

uint8 myEEPROM_1_storeConfig(void)    //store config buffer to EEPROM
{
    cystatus writeStatus = CYRET_BAD_PARAM;
    
    uint8 tmpBuf[SIZEOF_CONFIG_BUFFER+1];   //temporary copy of Config +CRC byte 
    uint8 * PtmpBuf = &(tmpBuf[0]);         //todo: make size multiple of 16 bytes?
    
    if (config_init_flag != 0) //if initialized correctly
    {
        
        //copy Config Buffer to tmpBuf and append CRC (CRC is last byte)
        uint16 index;  //buffer length may be upto 2048 bytes
        for(index = 0; index < SIZEOF_CONFIG_BUFFER; index++)
        {
            tmpBuf[index] = PConfig[index]; // copy Config to temp buffer
            //todo: check if faster: memcpy(tmpBuf, PConfig, SIZEOF_CONFIG_BUFFER)
        }
        tmpBuf[SIZEOF_CONFIG_BUFFER] = GetCRC8( PConfig ); //stick CRC to the end of tmpBuf
 
        
        myEEPROM_1_EEPROM_Start();
        CyDelay(5u); //EEPROM needs 5 µS to start, so no write requests should be performed. 
        
        if (myEEPROM_1_set_Temp_flag) CySetTemp(); //enable if temperature of operation may deviate for more than 10 degC
           
        //sprintf(strMsg1," StoreConfig: CRC=%d\r\n", tmpBuf[SIZEOF_CONFIG_BUFFER] );//debugging..
        //SW_Tx_UART_PutString(strMsg1);//debugging.. 
    
        //write config data to EEPROM raw-by-raw
        uint8 row_index = 0;
        while( row_index * SIZEOF_EEPROM_ROW < SIZEOF_CONFIG_BUFFER+1 ) 
        {
            //write 16 bytes from buffer to eeprom row 
            //todo: may have garbage beyond SIZEOF_CONFIG_BUFFER+1
            writeStatus = myEEPROM_1_EEPROM_Write(PtmpBuf + row_index * SIZEOF_EEPROM_ROW,
                                                        myEEPROM_1_EEPROM_CONFIG_LOCATION + row_index);
            if (writeStatus != CYRET_SUCCESS) break;
            row_index++;
        }
        
        myEEPROM_1_EEPROM_Stop();//optional
    
    }
    
    return (writeStatus==CYRET_SUCCESS);  
}


//===========================================================================

uint8 myEEPROM_1_Start (uint8 *ConfigPointer, uint16 value)
{    
    PConfig  = ConfigPointer;
    SIZEOF_CONFIG_BUFFER = value;//
    
    myEEPROM_1_CRC_SEED_VAL            = 0;
    myEEPROM_1_check_crc_flag          = 1;     
    myEEPROM_1_set_Temp_flag           = 0;
    myEEPROM_1_EEPROM_CONFIG_LOCATION  = 0;
   
    
    //check that Config buffer not empty and there is enough EEPROM space left to store it:
    config_init_flag = (PConfig != NULL)
                    && (SIZEOF_CONFIG_BUFFER   > 0) 
                    && (SIZEOF_CONFIG_BUFFER+1 <= (CYDEV_EE_SIZE - SIZEOF_EEPROM_ROW * myEEPROM_1_EEPROM_CONFIG_LOCATION));
     
    return config_init_flag;
}

//===========================================================================
void myEEPROM_1_Stop (void)
{    
    config_init_flag = 0;
    
    PConfig  = NULL;
    SIZEOF_CONFIG_BUFFER = 0; 
}

//===========================================================================
uint8 myEEPROM_1_Initialized (void)
{    
    return config_init_flag;
}
//===========================================================================
uint8 myEEPROM_1_SizeOfBuffer (void)
{    
    return SIZEOF_CONFIG_BUFFER;
}

//===========================================================================
/* [] END OF FILE */


